// handle.h v1.3.2
/*
 *  Copyright (c) 2010 Leigh Johnston.
 *
 *  All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions are
 *  met:
 *
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *
 *     * Neither the name of Leigh Johnston nor the names of any
 *       other contributors to this software may be used to endorse or
 *       promote products derived from this software without specific prior
 *       written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
 *  IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 *  THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 *  PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 *  CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 *  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 *  PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

#ifndef LIB_HANDLE
#define LIB_HANDLE

#include <windows.h>
#include <cstdlib>

namespace lib
{
	class handle_ref;

	class handle
	{
		// types
	private:
		typedef HANDLE os_handle;
		typedef DWORD os_timeout_interval;
	public:
		typedef os_timeout_interval timeout_interval;
		// constants
	public:
		static const os_handle HandleNone;
		static const os_handle HandleInvalid;
		static const timeout_interval TimeoutInfinite;
		static const timeout_interval TimeoutShort;
		// construction
	public:
		handle();
		handle(os_handle aHandle);
		handle(const handle& aOther);
		handle& operator=(const handle& aOther);
		handle& operator=(os_handle aHandle);
		virtual ~handle();
		// operations
	public:
		bool valid() const { return iHandle != HandleNone && iHandle != HandleInvalid; }
		operator bool() const { return valid(); }
		operator os_handle() const { return iHandle; }
		const handle* wait(timeout_interval aTimeoutInterval = TimeoutInfinite) const;
		const handle* msg_wait(timeout_interval aTimeoutInterval = TimeoutInfinite) const;
		const handle* wait_for_multiple(const handle& aHandle, timeout_interval aTimeoutInterval = TimeoutInfinite) const;
		const handle* msg_wait_for_multiple(const handle& aHandle, timeout_interval aTimeoutInterval = TimeoutInfinite) const;
		const handle* wait_for_multiple(const handle_ref* aHandleRefs, std::size_t aCount, timeout_interval aTimeoutInterval = TimeoutInfinite) const;
		const handle* msg_wait_for_multiple(const handle_ref* aHandleRefs = 0, std::size_t aCount = 0, timeout_interval aTimeoutInterval = TimeoutInfinite) const;
		void close();
		// implementation
	protected:
		void duplicate(const handle& aOther);
		// attributes
	protected:
		os_handle iHandle;
	};

	class handle_ref
	{
		// construction
	public:
		handle_ref(const handle& aHandle) : iHandle(&aHandle) {}
		// operations
	public:
		operator HANDLE() const { return *iHandle; }
		const handle* operator->() const { return iHandle; }
		const handle& operator*() const { return *iHandle; }
		// attributes
	private:
		const handle* iHandle;
	};
}

#endif //LIB_HANDLE
