// version.h
/*
 *  Copyright (c) 2010 Leigh Johnston.
 *
 *  All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions are
 *  met:
 *
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *
 *     * Neither the name of Leigh Johnston nor the names of any
 *       other contributors to this software may be used to endorse or
 *       promote products derived from this software without specific prior
 *       written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
 *  IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 *  THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 *  PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 *  CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 *  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 *  PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

#ifndef LIB_VERSION
#define LIB_VERSION

#include <string>
#include <ostream>
#include "vecarray.h"
#include "string.h"

namespace lib
{
	struct version
	{
		version(const std::string& aVersionString = "")
		{
			iValues[0] = iValues[1] = iValues[2] = iValues[3] = 0;
			lib::vecarray<std::string, 4> bits;
			lib::tokens(aVersionString, std::string("."), bits, 4);
			for (std::size_t i = 0; i < bits.size(); ++i)
				iValues[i] = lib::string_to_integer(bits[i]);
		}
		bool operator<(const version& aOther) const
		{
			for (std::size_t i = 0; i < 4; ++i)
				if (iValues[i] < aOther.iValues[i])
					return true;
				else if (iValues[i] > aOther.iValues[i])
					return false;
			return false;
		}
		bool operator>(const version& aOther) const
		{
			return aOther < *this;
		}
		bool operator==(const version& aOther) const
		{
			for (std::size_t i = 0; i < 4; ++i)
				if (iValues[i] != aOther.iValues[i])
					return false;
			return true;
		}
		int iValues[4];
	};
	inline std::ostream& operator<<(std::ostream& aStream, const version& aVersion)
	{
		aStream << aVersion.iValues[0] << "." << aVersion.iValues[1] << "." << aVersion.iValues[2] << "." << aVersion.iValues[3];
		return aStream;
	}
}

#endif //LIB_VERSION
