// win32_timer.cpp
/*
 *  Copyright (c) 2010 Leigh Johnston.
 *
 *  All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions are
 *  met:
 *
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *
 *     * Neither the name of Leigh Johnston nor the names of any
 *       other contributors to this software may be used to endorse or
 *       promote products derived from this software without specific prior
 *       written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
 *  IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 *  THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 *  PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 *  CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 *  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 *  PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

#include "stdafx.h"
#include "win32_timer.h"
#include <windows.h>

namespace lib
{
	namespace
	{
		LRESULT __stdcall tickerWndProc(HWND aWindow, UINT aMessage, WPARAM aWParam, LPARAM aLParam)
		{
			switch (aMessage)
			{
			case WM_NULL:
			case WM_TIMER:
				reinterpret_cast<win32_timer*>(GetWindowLongPtrA(aWindow, GWLP_USERDATA))->tick();
				return 0;
			default:
				return ::DefWindowProcA(aWindow, aMessage, aWParam, aLParam);
			}
		};
	}

	win32_timer::win32_timer(const std::string& aAutoTickerName, HINSTANCE aInstance) : iAutoTickerName("i42SoftwareTimerTicker::" + aAutoTickerName), iInstance(aInstance), iSink(NULL)
	{
		WNDCLASSEXA wcxNarrow;
		wcxNarrow.cbSize = sizeof(wcxNarrow);
		wcxNarrow.style = CS_PARENTDC|CS_GLOBALCLASS|CS_DBLCLKS;
		wcxNarrow.lpfnWndProc = tickerWndProc;
		wcxNarrow.cbClsExtra = 0;
		wcxNarrow.cbWndExtra = 0;
		wcxNarrow.hInstance = iInstance;
		wcxNarrow.hIcon = NULL;
		wcxNarrow.hCursor = NULL;
		wcxNarrow.hbrBackground = NULL;
		wcxNarrow.lpszMenuName = NULL;
		wcxNarrow.lpszClassName = iAutoTickerName.c_str();
		wcxNarrow.hIconSm = NULL;
		::RegisterClassExA(&wcxNarrow);
		iSink = ::CreateWindowA(iAutoTickerName.c_str(), "", WS_POPUP, 0, 0, 0, 0, NULL, NULL, iInstance, 0);
		::SetWindowLongPtrA(iSink, GWLP_USERDATA, (LONG_PTR)this);
		::SetTimer(iSink, 1, 10, NULL);
	}

	win32_timer::~win32_timer()
	{
		if (iSink != NULL)
		{
			::KillTimer(iSink, 1);
			::DestroyWindow(iSink);
			::UnregisterClassA(iAutoTickerName.c_str(), iInstance);
		}
	}

	unsigned long win32_timer::elapsed()
	{
		return ::GetTickCount();
	}
}
